@@ -52,7 +52,6 @@
 #include <ofono/phonebook.h>
 #include <ofono/sim.h>
 #include <ofono/sms.h>
-#include <ofono/ssn.h>
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 #include <ofono/stk.h>
@@ -170,7 +169,7 @@ static void simind_notify(GAtResult *result, gpointer user_data)
 	struct calypso_data *data = ofono_modem_get_data(modem);
 	GAtResultIter iter;
 
-	if (!data->sim)
+	if (data->sim == NULL)
 		return;
 
 	g_at_result_iter_init(&iter, result);
@@ -257,7 +256,7 @@ static void mux_setup(GAtMux *mux, gpointer user_data)
 
 	DBG("%p", mux);
 
-	if (!mux) {
+	if (mux == NULL) {
 		ofono_modem_set_powered(modem, FALSE);
 		return;
 	}
@@ -534,7 +533,6 @@ static void calypso_post_sim(struct ofono_modem *modem)
 				data->dlcs[NETREG_DLC]);
 	ofono_call_meter_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_barring_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
-	ofono_ssn_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_volume_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 
 	mw = ofono_message_waiting_create(modem);
