@@ -44,7 +44,9 @@
 #include <ofono/ussd.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
+#include <ofono/radio-settings.h>
 #include <ofono/log.h>
+#include <ofono/location-reporting.h>
 
 #include <drivers/atmodem/atutil.h>
 #include <drivers/atmodem/vendor.h>
@@ -64,6 +66,10 @@ struct mbm_data {
 	guint cpin_poll_source;
 	guint cpin_poll_count;
 	gboolean have_sim;
+	struct ofono_gprs *gprs;
+	struct ofono_gprs_context *gc;
+	struct ofono_location_reporting *lr;
+	guint reopen_source;
 	enum mbm_variant variant;
 };
 
@@ -74,7 +80,7 @@ static int mbm_probe(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	data = g_try_new0(struct mbm_data, 1);
-	if (!data)
+	if (data == NULL)
 		return -ENOMEM;
 
 	ofono_modem_set_data(modem, data);
@@ -88,6 +94,11 @@ static void mbm_remove(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	if (data->reopen_source > 0) {
+		g_source_remove(data->reopen_source);
+		data->reopen_source = 0;
+	}
+
 	ofono_modem_set_data(modem, NULL);
 
 	g_at_chat_unref(data->data_port);
@@ -103,7 +114,7 @@ static void mbm_debug(const char *str, void *user_data)
 {
 	const char *prefix = user_data;
 
-	ofono_info("%s %s", prefix, str);
+	ofono_info("%s%s", prefix, str);
 }
 
 static gboolean init_simpin_check(gpointer user_data);
@@ -162,11 +173,11 @@ static void mbm_quirk_d5530(struct ofono_modem *modem)
 				FALSE, NULL, NULL);
 }
 
-static void mbm_check_model(gboolean ok, GAtResult *result, gpointer user_data)
+static void check_model(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
 	GAtResultIter iter;
-	char const *model = "";
+	char const *model;
 
 	DBG("");
 
@@ -200,7 +211,7 @@ static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 	}
 
 	g_at_chat_send(data->modem_port, "AT+CGMM", NULL,
-			mbm_check_model, modem, NULL);
+					check_model, modem, NULL);
 }
 
 static void cfun_query(gboolean ok, GAtResult *result, gpointer user_data)
@@ -280,7 +291,7 @@ static GAtChat *create_port(const char *device)
 	GAtChat *chat;
 
 	channel = g_at_tty_open(device, NULL);
-	if (!channel)
+	if (channel == NULL)
 		return NULL;
 
 	syntax = g_at_syntax_new_gsm_permissive();
@@ -288,12 +299,67 @@ static GAtChat *create_port(const char *device)
 	g_at_syntax_unref(syntax);
 	g_io_channel_unref(channel);
 
-	if (!chat)
+	if (chat == NULL)
 		return NULL;
 
 	return chat;
 }
 
+static void mbm_disconnect(gpointer user_data);
+
+static gboolean reopen_callback(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct mbm_data *data = ofono_modem_get_data(modem);
+	const char *data_dev;
+
+	data->reopen_source = 0;
+
+	data_dev = ofono_modem_get_string(modem, "DataDevice");
+
+	data->data_port = create_port(data_dev);
+	if (data->data_port == NULL)
+		return FALSE;
+
+	if (getenv("OFONO_AT_DEBUG"))
+		g_at_chat_set_debug(data->data_port, mbm_debug, "Data: ");
+
+	g_at_chat_set_disconnect_function(data->data_port,
+						mbm_disconnect, modem);
+
+	ofono_info("Reopened GPRS context channel");
+
+	data->gc = ofono_gprs_context_create(modem, 0,
+					"atmodem", data->data_port);
+	if (data->gprs && data->gc) {
+		ofono_gprs_context_set_type(data->gc,
+					OFONO_GPRS_CONTEXT_TYPE_MMS);
+		ofono_gprs_add_context(data->gprs, data->gc);
+	}
+
+	return FALSE;
+}
+
+static void mbm_disconnect(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct mbm_data *data = ofono_modem_get_data(modem);
+
+	DBG("");
+
+	if (data->gc)
+		ofono_gprs_context_remove(data->gc);
+
+	g_at_chat_unref(data->data_port);
+	data->data_port = NULL;
+
+	/* Waiting for the +CGEV: ME DEACT might also work */
+	if (data->reopen_source > 0)
+		g_source_remove(data->reopen_source);
+
+	data->reopen_source = g_timeout_add_seconds(1, reopen_callback, modem);
+}
+
 static int mbm_enable(struct ofono_modem *modem)
 {
 	struct mbm_data *data = ofono_modem_get_data(modem);
@@ -311,15 +377,13 @@ static int mbm_enable(struct ofono_modem *modem)
 		return -EINVAL;
 
 	data->modem_port = create_port(modem_dev);
-
 	if (data->modem_port == NULL)
 		return -EIO;
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->modem_port, mbm_debug, "Modem:");
+		g_at_chat_set_debug(data->modem_port, mbm_debug, "Modem: ");
 
 	data->data_port = create_port(data_dev);
-
 	if (data->data_port == NULL) {
 		g_at_chat_unref(data->modem_port);
 		data->modem_port = NULL;
@@ -328,13 +392,19 @@ static int mbm_enable(struct ofono_modem *modem)
 	}
 
 	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(data->data_port, mbm_debug, "Data:");
+		g_at_chat_set_debug(data->data_port, mbm_debug, "Data: ");
+
+	g_at_chat_set_disconnect_function(data->data_port,
+						mbm_disconnect, modem);
 
 	g_at_chat_register(data->modem_port, "*EMRDY:", emrdy_notifier,
 					FALSE, modem, NULL);
 
 	g_at_chat_send(data->modem_port, "AT&F E0 V1 X4 &C1 +CMEE=1", NULL,
 					NULL, NULL, NULL);
+	g_at_chat_send(data->data_port, "AT&F E0 V1 X4 &C1 +CMEE=1", NULL,
+					NULL, NULL, NULL);
+
 	g_at_chat_send(data->modem_port, "AT*E2CFUN=1", none_prefix,
 					NULL, NULL, NULL);
 	g_at_chat_send(data->modem_port, "AT*EMRDY?", none_prefix,
@@ -366,7 +436,12 @@ static int mbm_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	if (!data->modem_port)
+	if (data->reopen_source > 0) {
+		g_source_remove(data->reopen_source);
+		data->reopen_source = 0;
+	}
+
+	if (data->modem_port == NULL)
 		return 0;
 
 	g_at_chat_cancel_all(data->modem_port);
@@ -398,13 +473,9 @@ static void mbm_set_online(struct ofono_modem *modem, ofono_bool_t online,
 
 	DBG("modem %p %s", modem, online ? "online" : "offline");
 
-	if (!cbd)
-		goto error;
-
 	if (g_at_chat_send(chat, command, NULL, set_online_cb, cbd, g_free))
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
@@ -432,38 +503,58 @@ static void mbm_post_sim(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_stk_create(modem, 0, "mbmmodem", data->modem_port);
+	ofono_radio_settings_create(modem, 0, "stemodem", data->modem_port);
+
+	ofono_sms_create(modem, 0, "atmodem", data->modem_port);
 }
 
 static void mbm_post_online(struct ofono_modem *modem)
 {
 	struct mbm_data *data = ofono_modem_get_data(modem);
-	struct ofono_gprs *gprs;
 	struct ofono_gprs_context *gc;
+	const char *gps_dev;
 
 	DBG("%p", modem);
 
+	gps_dev = ofono_modem_get_string(modem, "GPSDevice");
+	if (gps_dev)
+		data->lr = ofono_location_reporting_create(modem, 0,
+					"mbmmodem", data->modem_port);
+
 	ofono_netreg_create(modem, OFONO_VENDOR_MBM,
 					"atmodem", data->modem_port);
 
-	ofono_sms_create(modem, 0, "atmodem", data->modem_port);
-
 	switch (data->variant) {
+	case MBM_GENERIC:
+		ofono_cbs_create(modem, 0, "atmodem", data->modem_port);
+		break;
 	case MBM_DELL_D5530:
 		/* DELL D5530 crashes when it processes CBSs */
 		break;
-	default:
-		ofono_cbs_create(modem, 0, "atmodem", data->modem_port);
 	}
 
 	ofono_ussd_create(modem, 0, "atmodem", data->modem_port);
 
-	gprs = ofono_gprs_create(modem, OFONO_VENDOR_MBM,
+	data->gprs = ofono_gprs_create(modem, OFONO_VENDOR_MBM,
 					"atmodem", data->modem_port);
+	if (data->gprs == NULL)
+		return;
+
 	gc = ofono_gprs_context_create(modem, 0,
 					"mbmmodem", data->modem_port);
+	if (gc) {
+		ofono_gprs_context_set_type(gc,
+					OFONO_GPRS_CONTEXT_TYPE_INTERNET);
+		ofono_gprs_add_context(data->gprs, gc);
+	}
 
-	if (gprs && gc)
-		ofono_gprs_add_context(gprs, gc);
+	data->gc = ofono_gprs_context_create(modem, 0,
+					"atmodem", data->data_port);
+	if (data->gc) {
+		ofono_gprs_context_set_type(data->gc,
+					OFONO_GPRS_CONTEXT_TYPE_MMS);
+		ofono_gprs_add_context(data->gprs, data->gc);
+	}
 }
 
 static struct ofono_modem_driver mbm_driver = {
